#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
# from django.core.management.base import CommandError
from optparse import make_option
from django.conf import settings
import os
import sys
#import yaml
# from boto.s3.connection import S3Connection
# from boto.exception import S3ResponseError
# from boto.s3.key import Key
from subprocess import Popen, PIPE
# import logging
from time import sleep, time

# log = logging.getLogger(__name__)
"""
Django management command to dump database content and then upload to S3
TODO: Load from config file, refactor. We have hardcoded value of parent object
      and it does not match base object or anything: storage_bucket? file is
      storage_bucket.yaml, devops decided that.
TODO: Rename upload_manager to progress_manager
TODO: Should we be using Popen?
TODO: Get/handle errors from command.
TODO: Decouple action shell commands, use from template?
TODO: Should we name file dumps with DDBB type?
DONE: Rename to dumpdbS3 or something that reflects the fact that is stored
      in S3.
DONE: Do we want to keep a copy? If so, then we need to/could do two uploads,
      one with the regular file name, the other with a timestamp. So only the
      regular name would get overridden and we could always pull the latest by
      name.
"""

HELP = ['Takes a snapshot of the database and upload to S3',
        'Usage:',
        "\t\t\tpython manage.py dumpdbs3 -d default -b dump-test -i",
        "\t\t\tpython manage.py dumpdbs3 -m create -b creates-this-bucket",
        'Config:',
        "With the -C option we can take configuration from a config file."
        ]


class Command(BaseCommand):

    def add_arguments(self,parser):
        parser.add_argument('-d', '--database', dest="database",default='default',help='Database router id.')
        parser.add_argument('-i', '--incremental', action="store_true",help="Keep an incremental build of all dumps, "
                 "keeping the head always as the latest.")

    def handle(self, *args, **options):

        self.file_name = 'dump.dat'
        self.database = options.get('database') or ''
        database_keys = self.database.split(',') or settings.DATABASES
        

        self.stdout.write("Execute dumpdb command: %s " % self.target)

        # Start initial dump.
        incremental = options['incremental']

        self.start_dump(incremental)

    def load_config(self, path):
        file = open(path, 'r')
        config = file
        # TODO: We have this hardcoded, we should fix it!
        # TODO: This is weak! We need to check for stuff,
        #       catch errors and what nots!!
        config = config['storage_bucket']
        for name in config:
            setattr(self, name, config[name])
        file.close()

    def start_dump(self, incremental):
        dump = self.dump_database()
        if not dump:
            # This usually means that we got an error on the dump process
            # ie: unknown postgress user.
            print("We got not data from dump. Exiting with error.")
            sys.exit(0)

        # We store state, using incremental to track first/second
        # pass. It will affect how we name our dump file.
        self.incremental = incremental

        self.file_path = self.get_file_path(incremental)

        self.save_dump_to_file(self.file_path, dump)

        self.upload_dump_to_S3(self.file_path,
                               self.bucket_name, self.bucket_factory)

    def get_file_path(self, incremental=True):
        # timestamp = incremental and str(time()).split('.')[0] or ""
        timestamp = str(time()).split('.')[0] + '_' if incremental else ""
        return './%s%s' % (timestamp, self.file_name)

    def dump_database(self):
        print("Dump output ready...")
        dumper_method = self.get_dump_method()
        return dumper_method(self.database_settings['NAME'])

    def get_dump_method(self):
        # TODO: Abstract getting settings to method!
        database_engine = self.database_settings['ENGINE']

        # get the right dumper based on DDBB
        dumper_type = self.get_dumper_type(database_engine)
        return getattr(self, dumper_type)

    def save_dump_to_file(self, file_name, dump):
        print("Create dump file...")
        file = open(file_name, 'w+')
        file.write(dump)
        file.close()

    #def upload_dump_to_S3(self, file_name, bucket_name, bucket_factory='get'):
        # print("Pushing %s to S3: %s..." % (file_name, bucket_name))

        # # Sanitize bucket factory method
        # if bucket_factory not in ("get", "create"):
        #     bucket_factory = 'get'

        # conn = S3Connection()

        # try:
        #     # This user requires create permissions, we could just
        #     # change it to use get_bucket if it already exists...
        #     # TODO: Implement command option to use dif method.
        #     bucket_factory = getattr(conn, '%s_bucket' % bucket_factory)
        #     bucket = bucket_factory(bucket_name)
        # except S3ResponseError as e:
        #     self.stdout.write("Error while trying to connect to S3.\n%s" % e)
        #     sys.exit(1)

        # manager = self.upload_manager(self.upload_done)

        # k = Key(bucket)
        # k.key = os.path.basename(file_name)
        # k.set_contents_from_filename(file_name,
        #                              cb=manager, num_cb=20)

    def get_postgresql_psycopg2_dump(self, database):
        # Generate dump file with droptables and all data.
        params = ["sudo", "-u", "postgres",
                  "pg_dump", "-c", database]
        return self.execute_dump(params)

    def get_mysql_dump(self, database):
        params = ['mysqldump', '--user=root',
                  database]
        return self.execute_dump(params)

    def get_sqlite3_dump(self, database):
        pass
        params = ['sqlite3', database,
                  '.dump']
        return self.execute_dump(params)

    def execute_dump(self, params):
        process = Popen(params, stdout=PIPE)
        output = process.communicate()[0]
        return output

    def upload_manager(self, on_done):
        """
        Wrap the progress method so that we can bundle a callback
        on done. We need this because we can't append the callback
        in the progress callback.
        """
        def _upload_progress(loaded=0, total=0, delay=0.4):
            percent = int((loaded*100)/total)
            remainder = 100 - percent
            output = 'Uploading: [{0}{1}] {2}%\r'.format('#' * percent,
                                                         ' ' * remainder,
                                                         percent)
            if percent == 100:
                output = output + '\n'
                delay = 0
            sys.stderr.write(output)
            sleep(delay)
            # We are done!
            if percent == 100:
                on_done()

        return _upload_progress

    def upload_done(self):
        # get the file name and make path
        os.remove(self.file_path)

        if self.incremental:
            print("Uploaded first file.")
            self.start_dump(False)
        else:
            print("Cleaning up. Done!")

    def get_dumper_type(self, engine):
        """
        django.db.backends.postgresql_psycopg2:  psql
        django.db.backends.mysql              :  mysql
        django.db.backends.sqlite3            :  sqlite3
        """
        p = "django.db.backends."
        return 'get_%s_dump' % engine.replace(p, "")


import os
import sys
from subprocess import Popen, PIPE
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from ...dbconnect.base import get_connector


class StorageError(Exception):
    pass

class Command(BaseCommand):
    """
    docstring
    """
    def add_arguments(self,parser):
        #parser.add_argument('-d', '--database', action="store_true", help='to dump default primary database')
        parser.add_argument('-s', '--servername', help='servername')

    def handle(self, *args, **options):
        self.servername = options.get('servername')
        database_keys = settings.DATABASES
        
        for database_key in database_keys:
            self.connector = get_connector(database_key)
            database = self.connector.settings
            try:
                self._save_new_backup(database)
            except StorageError as err:
                print(err)

    def _save_new_backup(self, database):
        """
        Save a new backup file.
        """
        print("Backing Up Database:", database['NAME'])
        # Get backup and name
        filename = self.connector.generate_filename(self.servername)
        outputfile = self.connector.create_dump()
        # Apply trans
        # Set file name
        filename = filename
        # Store backup
        outputfile.seek(0)
        self.connector.write_local_file(outputfile, filename)


from django.core.management.base import BaseCommand

class Command(BaseCommand):
    help = 'Show all available styles'

    def handle(self, *args, **kwargs):
        self.stdout.write(self.style.ERROR('error - A major error.'))
        self.stdout.write(self.style.NOTICE('notice - A minor error.'))
        self.stdout.write(self.style.SUCCESS('success - A success.'))
        self.stdout.write(self.style.WARNING('warning - A warning.'))
        self.stdout.write(self.style.SQL_FIELD('sql_field - The name of a model field in SQL.'))
        self.stdout.write(self.style.SQL_COLTYPE('sql_coltype - The type of a model field in SQL.'))
        self.stdout.write(self.style.SQL_KEYWORD('sql_keyword - An SQL keyword.'))
        self.stdout.write(self.style.SQL_TABLE('sql_table - The name of a model in SQL.'))
        self.stdout.write(self.style.HTTP_INFO('http_info - A 1XX HTTP Informational server response.'))
        self.stdout.write(self.style.HTTP_SUCCESS('http_success - A 2XX HTTP Success server response.'))
        self.stdout.write(self.style.HTTP_NOT_MODIFIED('http_not_modified - A 304 HTTP Not Modified server response.'))
        self.stdout.write(self.style.HTTP_REDIRECT('http_redirect - A 3XX HTTP Redirect server response other than 304.'))
        self.stdout.write(self.style.HTTP_NOT_FOUND('http_not_found - A 404 HTTP Not Found server response.'))
        self.stdout.write(self.style.HTTP_BAD_REQUEST('http_bad_request - A 4XX HTTP Bad Request server response other than 404.'))
        self.stdout.write(self.style.HTTP_SERVER_ERROR('http_server_error - A 5XX HTTP Server Error response.'))
        self.stdout.write(self.style.MIGRATE_HEADING('migrate_heading - A heading in a migrations management command.'))
        self.stdout.write(self.style.MIGRATE_LABEL('migrate_label - A migration name.'))


from django.core.management.base import BaseCommand
from django.contrib.auth.models import User

class Command(BaseCommand):
    help = 'helps to delete specified user ids'
    def add_arguments(self,parser):
        #arbitary list arguments
        parser.add_argument('user_id', nargs='+', type=int, help='enter valid user id')

    def handle(self, *args, **options):
        """
        delete mentioned user id's
        """
        user_ids = options['user_id']

        for uid in user_ids:
            try:
                user = User.objects.get(pk=uid)
                user.delete()
                self.stdout.write('User "%s (%s)" deleted successfully' %(user.username, uid))
            except User.DoesNotExist:
                self.stderr.write('User id "%s" does not exist' %(uid))


from .base import BaseCommandDBConnector


class MysqlDumpConnector(BaseCommandDBConnector):
    """
    MySQL connector, creates dump with ``mysqldump`` and restore with
    ``mysql``.
    """
    dump_cmd = 'mysqldump'
    restore_cmd = 'mysql'

    def _create_dump(self):
        cmd = '{} {} --quick'.format(self.dump_cmd, self.settings['NAME'])
        if self.settings.get('HOST'):
            cmd += ' --host={}'.format(self.settings['HOST'])
        if self.settings.get('PORT'):
            cmd += ' --port={}'.format(self.settings['PORT'])
        if self.settings.get('USER'):
            cmd += ' --user={}'.format(self.settings['USER'])
        if self.settings.get('PASSWORD'):
            cmd += ' --password={}'.format(self.settings['PASSWORD'])
        for table in self.exclude:
            cmd += ' --ignore-table={}.{}'.format(self.settings['NAME'], table)
        cmd = '{} {} {}'.format(self.dump_prefix, cmd, self.dump_suffix)
        stdout, stderr = self.run_command(cmd, env=self.dump_env)
        return stdout

    def _restore_dump(self, dump):
        cmd = '{} {}'.format(self.restore_cmd, self.settings['NAME'])
        if self.settings.get('HOST'):
            cmd += ' --host={}'.format(self.settings['HOST'])
        if self.settings.get('PORT'):
            cmd += ' --port={}'.format(self.settings['PORT'])
        if self.settings.get('USER'):
            cmd += ' --user={}'.format(self.settings['USER'])
        if self.settings.get('PASSWORD'):
            cmd += ' --password={}'.format(self.settings['PASSWORD'])
        cmd = '{} {} {}'.format(self.restore_prefix, cmd, self.restore_suffix)
        stdout, stderr = self.run_command(cmd, stdin=dump, env=self.restore_env)
        return stdout, stderr



import tempfile
from django.conf import settings

TMP_FILE_MAX_SIZE = getattr(settings, 'DBBACKUP_TMP_FILE_MAX_SIZE', 10 * 1024 * 1024)
TMP_FILE_READ_SIZE = getattr(settings, 'DBBACKUP_TMP_FILE_READ_SIZE', 1024 * 1000)

CONNECTORS = getattr(settings, 'DATABASES', {})
DATABASES = getattr(settings, 'DATABASES', list(settings.DATABASES.keys()))
TMP_DIR = getattr(settings, 'DBDUMP_DIR', tempfile.gettempdir())


import os
import shlex
from django.core.files.base import File
from tempfile import SpooledTemporaryFile
from subprocess import Popen
from importlib import import_module
from backupdb import settings
from shutil import copyfileobj

CONNECTOR_MAPPING = {
    'django.db.backends.sqlite3': 'backupdb.dbconnect.sqlite.SqliteConnector',
    'django.db.backends.mysql': 'backupdb.dbconnect.mysql.MysqlDumpConnector',
    'django.db.backends.postgresql': 'backupdb.dbconnect.postgresql.PgDumpConnector',
    'django.db.backends.postgresql_psycopg2': 'backupdb.dbconnect.postgresql.PgDumpConnector',
    'django.db.backends.oracle': None,
    'django_mongodb_engine': 'backupdb.dbconnect.mongodb.MongoDumpConnector',
    'djongo': 'backupdb.dbconnect.mongodb.MongoDumpConnector',
    'django.contrib.gis.db.backends.postgis': 'backupdb.dbconnect.postgresql.PgDumpGisConnector',
    'django.contrib.gis.db.backends.mysql': 'backupdb.dbconnect.mysql.MysqlDumpConnector',
    'django.contrib.gis.db.backends.oracle': None,
    'django.contrib.gis.db.backends.spatialite': 'backupdb.dbconnect.sqlite.SqliteConnector',
}



def get_connector(database_name=None):
    """
    Get a connector from its database key in setttings.
    """
    from django.db import connections, DEFAULT_DB_ALIAS
    # Get DB
    database_name = database_name or DEFAULT_DB_ALIAS
    connection = connections[database_name]
    engine = connection.settings_dict['ENGINE']
    connector_settings = settings.CONNECTORS.get(database_name, {})
    connector_path = connector_settings.get('CONNECTOR', CONNECTOR_MAPPING[engine])
    connector_module_path = '.'.join(connector_path.split('.')[:-1])
    module = import_module(connector_module_path)
    connector_name = connector_path.split('.')[-1]
    connector = getattr(module, connector_name)
    return connector(database_name, **connector_settings)

class BaseDBConnector(object):
    """
    Base class for create database connector. This kind of object creates
    interaction with database and allow backup and restore operations.
    """
    extension = 'dump'
    exclude = []

    def __init__(self, database_name=None, **kwargs):
        from django.db import connections, DEFAULT_DB_ALIAS
        self.database_name = database_name or DEFAULT_DB_ALIAS
        self.connection = connections[self.database_name]
        for attr, value in kwargs.items():
            setattr(self, attr.lower(), value)
    
    @property
    def settings(self):
        """Mix of database and connector settings."""
        if not hasattr(self, '_settings'):
            sett = self.connection.settings_dict.copy()
            sett.update(settings.DATABASES.get(self.database_name, {}))
            self._settings = sett
        return self._settings

    def generate_filename(self, server_name=None):
        return self.name+'.'+self.extension

    def create_dump(self):
        dump = self._create_dump()
        return dump

    def write_local_file(self, outputfile, path):
        """Write file to the desired path."""
        print("Writing file to ", path)
        outputfile.seek(0)
        with open(path, 'wb') as fd:
            copyfileobj(outputfile, fd)
        
class BaseCommandDBConnector(BaseDBConnector):
    """
    Base class for create database connector based on command line tools.
    """
    dump_prefix = ''
    dump_suffix = ''
    restore_prefix = ''
    restore_suffix = ''

    use_parent_env = True
    env = {}
    dump_env = {}
    restore_env = {}

    def run_command(self, command, stdin=None, env=None):
        """
        Launch a shell command line.

        :param command: Command line to launch
        :type command: str
        :param stdin: Standard input of command
        :type stdin: file
        :param env: Environment variable used in command
        :type env: dict
        :return: Standard output of command
        :rtype: file
        """
        cmd = shlex.split(command)
        stdout = SpooledTemporaryFile(max_size=settings.TMP_FILE_MAX_SIZE,
                                      dir=settings.TMP_DIR)
        stderr = SpooledTemporaryFile(max_size=settings.TMP_FILE_MAX_SIZE,
                                      dir=settings.TMP_DIR)
        full_env = os.environ.copy() if self.use_parent_env else {}
        full_env.update(self.env)
        full_env.update(env or {})
        try:
            if isinstance(stdin, File):
                process = Popen(
                    cmd, stdin=stdin.open("rb"), stdout=stdout, stderr=stderr,
                    env=full_env
                )
            else:
                process = Popen(cmd, stdin=stdin, stdout=stdout, stderr=stderr, env=full_env)
            process.wait()
            if process.poll():
                stderr.seek(0)
                self.stdout.write("Error running:")
            stdout.seek(0)
            stderr.seek(0)
            return stdout, stderr
        except OSError as err:
            self.stdout.write("Error running 1")



from setuptools import setup, find_packages
 
classifiers = [
  'Development Status :: 4 - Beta',
  'Intended Audience :: Education',
  'Operating System :: POSIX :: Linux',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3'
]
 
setup(
  name='mysqldump',
  version='0.0.3',
  description='Mysqldump is a django package that is used to generate the logical backup of the MySQL database',
  long_description=open('README.txt').read() + '\n\n' + open('CHANGELOG.txt').read(),
  url='',  
  author='Nandhakumar D',
  author_email='dnandhakumars@gmail.com',
  license='MIT', 
  classifiers=classifiers,
  keywords='mysqldump', 
  packages=find_packages(),
  install_requires=['django>=3.0'] 
)


from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from django.utils.crypto import get_random_string


class Command(BaseCommand):
    help = "Executed the the backupdb command"

    def add_arguments(self, parser):
        #positional argument
        parser.add_argument('total', type=int, help='number of user to be created')
        #optional arguments
        parser.add_argument('-p', '--prefix', type=str, help="optional argument -p add username prefix")
        #flag arguments to set boolean value
        parser.add_argument('-a', '--admin', action='store_true', help='to create user as admin')


    def handle(self, *args, **options):
        self.stdout.write("cmd started")
        total = options['total']
        prefix = options['prefix']
        admin = options['admin']

        for i in range(total):
            if prefix:
                username = '{prefix}_{random_string}'.format(prefix=prefix, random_string=get_random_string())
            else:
                username = get_random_string()
                
            if admin:
                User.objects.create_superuser(username=username, email='', password = '123456789')
            else:
                User.objects.create_user(username=username, email='', password = '123456789')
        self.stdout.write("CMD executed successfully")